@@ -41,12 +41,8 @@
 //M*/
 
 #include "test_precomp.hpp"
-#include "opencv2/calib3d/calib3d_c.h"
-#include <string>
-#include <limits>
 
-using namespace cv;
-using namespace std;
+namespace opencv_test { namespace {
 
 template<class T> double thres() { return 1.0; }
 template<> double thres<float>() { return 1e-5; }
@@ -124,14 +120,9 @@ protected:
 
         Mat_<double> Q(4, 4);
         randu(Q, Scalar(-5), Scalar(5));
-
         Mat_<out3d_t> _3dImg(disp.size());
 
-        CvMat cvdisp = disp; CvMat cv_3dImg = _3dImg; CvMat cvQ = Q;
-        cvReprojectImageTo3D( &cvdisp, &cv_3dImg, &cvQ, handleMissingValues );
-
-        if (numeric_limits<OutT>::max() == numeric_limits<float>::max())
-            reprojectImageTo3D(disp, _3dImg, Q, handleMissingValues);
+        reprojectImageTo3D(disp, _3dImg, Q, handleMissingValues);
 
         for(int y = 0; y < disp.rows; ++y)
             for(int x = 0; x < disp.cols; ++x)
@@ -178,3 +169,5 @@ protected:
 };
 
 TEST(Calib3d_ReprojectImageTo3D, accuracy) { CV_ReprojectImageTo3DTest test; test.safe_run(); }
+
+}} // namespace
